@@ -14,7 +14,9 @@
 #include "stl_util.h"
 
 #include "libkleo_debug.h"
+#include "utils/algorithm.h"
 #include "utils/formatting.h"
+#include "utils/gnupg.h"
 
 #include <KConfig>
 #include <KConfigGroup>
@@ -139,13 +141,16 @@ public:
     KeyNotValidFilter()
         : DefaultKeyFilter()
     {
-        setName(i18n("Not validated Certificates"));
-        setId(QStringLiteral("not-validated-certificates"));
         setSpecificity(UINT_MAX - 6); // overly high for ordering
+
+        setName(i18n("Not Validated Certificates"));
+        setId(QStringLiteral("not-validated-certificates"));
+        setMatchContexts(Filtering);
     }
     bool matches (const Key &key, MatchContexts contexts) const override
     {
-        return (contexts & Filtering) && !Formatting::uidsHaveFullValidity(key);
+        return DefaultKeyFilter::matches(key, contexts)
+            && !Formatting::uidsHaveFullValidity(key);
     }
 };
 
@@ -167,18 +172,17 @@ static std::vector<std::shared_ptr<KeyFilter>> defaultFilters()
 class KeyFilterManager::Private
 {
 public:
-    Private() : filters(), appearanceFilters(), model(this) {}
+    Private() : filters(), model(this) {}
     void clear()
     {
         model.beginResetModel();
         filters.clear();
-        appearanceFilters.clear();
         model.endResetModel();
     }
 
     std::vector<std::shared_ptr<KeyFilter>> filters;
-    std::vector<std::shared_ptr<KeyFilter>> appearanceFilters;
     Model model;
+    GpgME::Protocol protocol = GpgME::UnknownProtocol;
 };
 
 KeyFilterManager *KeyFilterManager::mSelf = nullptr;
@@ -200,7 +204,6 @@ KeyFilterManager::~KeyFilterManager()
     if (d) {
         d->clear();
     }
-    delete d; d = nullptr;
 }
 
 KeyFilterManager *KeyFilterManager::instance()
@@ -211,6 +214,14 @@ KeyFilterManager *KeyFilterManager::instance()
     return mSelf;
 }
 
+void KeyFilterManager::alwaysFilterByProtocol(GpgME::Protocol protocol)
+{
+    if (protocol != d->protocol) {
+        d->protocol = protocol;
+        reload();
+    }
+}
+
 const std::shared_ptr<KeyFilter> &KeyFilterManager::filterMatching(const Key &key, KeyFilter::MatchContexts contexts) const
 {
     const auto it = std::find_if(d->filters.cbegin(), d->filters.cend(),
@@ -254,7 +265,7 @@ void KeyFilterManager::reload()
     KSharedConfigPtr config = KSharedConfig::openConfig(QStringLiteral("libkleopatrarc"));
 
     const QStringList groups = config->groupList().filter(QRegularExpression(QStringLiteral("^Key Filter #\\d+$")));
-    bool ignoreDeVs = Formatting::complianceMode() != QLatin1String("de-vs");
+    const bool ignoreDeVs = !Kleo::gnupgIsDeVsCompliant();
     for (QStringList::const_iterator it = groups.begin(); it != groups.end(); ++it) {
         const KConfigGroup cfg(config, *it);
         if (cfg.hasKey("is-de-vs") && ignoreDeVs) {
@@ -264,7 +275,26 @@ void KeyFilterManager::reload()
         d->filters.push_back(std::shared_ptr<KeyFilter>(new KConfigBasedKeyFilter(cfg)));
     }
     std::stable_sort(d->filters.begin(), d->filters.end(), ByDecreasingSpecificity());
-    qCDebug(LIBKLEO_LOG) << "final filter count is" << d->filters.size();
+
+    if (d->protocol != GpgME::UnknownProtocol) {
+        // remove filters with conflicting isOpenPGP rule
+        const auto conflictingValue = (d->protocol == GpgME::OpenPGP) ? DefaultKeyFilter::NotSet : DefaultKeyFilter::Set;
+        Kleo::erase_if(d->filters,
+                       [conflictingValue](const auto &f) {
+                           const auto filter = std::dynamic_pointer_cast<DefaultKeyFilter>(f);
+                           Q_ASSERT(filter);
+                           return filter->isOpenPGP() == conflictingValue;
+                       });
+        // add isOpenPGP rule to all filters
+        const auto isOpenPGPValue = (d->protocol == GpgME::OpenPGP) ? DefaultKeyFilter::Set : DefaultKeyFilter::NotSet;
+        std::for_each(std::begin(d->filters), std::end(d->filters),
+                      [isOpenPGPValue](auto &f) {
+                           const auto filter = std::dynamic_pointer_cast<DefaultKeyFilter>(f);
+                           Q_ASSERT(filter);
+                           return filter->setIsOpenPGP(isOpenPGPValue);
+                      });
+    }
+    qCDebug(LIBKLEO_LOG) << "KeyFilterManager::" << __func__ << "final filter count is" << d->filters.size();
 }
 
 QAbstractItemModel *KeyFilterManager::model() const
@@ -333,9 +363,12 @@ QVariant Model::data(const QModelIndex &idx, int role) const
     case Qt::ToolTipRole:  /* Most useless tooltip ever.  */
         return filter->name();
 
-    case Qt::UserRole:
+    case KeyFilterManager::FilterIdRole:
         return filter->id();
 
+    case KeyFilterManager::FilterMatchContextsRole:
+        return QVariant::fromValue(filter->availableMatchContexts());
+
     default:
         return QVariant();
     }
@@ -356,10 +389,7 @@ static KeyFilter::FontDescription get_fontdescription(const std::vector<std::sha
 
 QFont KeyFilterManager::font(const Key &key, const QFont &baseFont) const
 {
-    KeyFilter::FontDescription fd;
-
-    fd = get_fontdescription(d->appearanceFilters, key, KeyFilter::FontDescription());
-    fd = get_fontdescription(d->filters, key, fd);
+    const KeyFilter::FontDescription fd = get_fontdescription(d->filters, key, KeyFilter::FontDescription());
 
     return fd.font(baseFont);
 }
@@ -394,34 +424,16 @@ static QString get_string(const std::vector<std::shared_ptr<KeyFilter>> &filters
 
 QColor KeyFilterManager::bgColor(const Key &key) const
 {
-    QColor color;
-
-    color = get_color(d->appearanceFilters, key, &KeyFilter::bgColor);
-    if (!color.isValid()) {
-        color = get_color(d->filters, key, &KeyFilter::bgColor);
-    }
-    return color;
+    return get_color(d->filters, key, &KeyFilter::bgColor);
 }
 
 QColor KeyFilterManager::fgColor(const Key &key) const
 {
-    QColor color;
-
-    color = get_color(d->appearanceFilters, key, &KeyFilter::fgColor);
-    if (!color.isValid()) {
-        color = get_color(d->filters, key, &KeyFilter::fgColor);
-    }
-    return color;
+    return get_color(d->filters, key, &KeyFilter::fgColor);
 }
 
 QIcon KeyFilterManager::icon(const Key &key) const
 {
-    QString icon;
-
-    icon = get_string(d->appearanceFilters, key, &KeyFilter::icon);
-    if (icon.isEmpty()) {
-        icon = get_string(d->filters, key, &KeyFilter::icon);
-    }
+    const QString icon = get_string(d->filters, key, &KeyFilter::icon);
     return icon.isEmpty() ? QIcon() : QIcon::fromTheme(icon);
 }
-
