@@ -3,7 +3,7 @@
 
     This file is part of Kleopatra, the KDE keymanager
     SPDX-FileCopyrightText: 2007 Klarälvdalens Datakonsult AB
-    SPDX-FileCopyrightText: 2021 g10 Code GmbH
+    SPDX-FileCopyrightText: 2021, 2022 g10 Code GmbH
     SPDX-FileContributor: Ingo Klöcker <dev@ingo-kloecker.de>
 
     SPDX-License-Identifier: GPL-2.0-or-later
@@ -17,6 +17,7 @@
 #include "kleo/keygroup.h"
 
 #include "utils/cryptoconfig.h"
+#include "utils/gnupg.h"
 
 #include <gpgme++/key.h>
 #include <gpgme++/importresult.h>
@@ -46,7 +47,7 @@ using namespace Kleo;
 QString Formatting::prettyName(int proto, const char *id, const char *name_, const char *comment_)
 {
 
-    if (proto == OpenPGP) {
+    if (proto == GpgME::OpenPGP) {
         const QString name = QString::fromUtf8(name_);
         if (name.isEmpty()) {
             return QString();
@@ -58,7 +59,7 @@ QString Formatting::prettyName(int proto, const char *id, const char *name_, con
         return QStringLiteral("%1 (%2)").arg(name, comment);
     }
 
-    if (proto == CMS) {
+    if (proto == GpgME::CMS) {
         const DN subject(id);
         const QString cn = subject[QStringLiteral("CN")].trimmed();
         if (cn.isEmpty()) {
@@ -78,7 +79,7 @@ QString Formatting::prettyNameAndEMail(int proto, const char *id, const char *na
 QString Formatting::prettyNameAndEMail(int proto, const QString &id, const QString &name, const QString &email, const QString &comment)
 {
 
-    if (proto == OpenPGP) {
+    if (proto == GpgME::OpenPGP) {
         if (name.isEmpty()) {
             if (email.isEmpty()) {
                 return QString();
@@ -102,7 +103,7 @@ QString Formatting::prettyNameAndEMail(int proto, const QString &id, const QStri
         }
     }
 
-    if (proto == CMS) {
+    if (proto == GpgME::CMS) {
         const DN subject(id);
         const QString cn = subject[QStringLiteral("CN")].trimmed();
         if (cn.isEmpty()) {
@@ -115,7 +116,7 @@ QString Formatting::prettyNameAndEMail(int proto, const QString &id, const QStri
 
 QString Formatting::prettyUserID(const UserID &uid)
 {
-    if (uid.parent().protocol() == OpenPGP) {
+    if (uid.parent().protocol() == GpgME::OpenPGP) {
         return prettyNameAndEMail(uid);
     }
     const QByteArray id = QByteArray(uid.id()).trimmed();
@@ -161,7 +162,7 @@ QString Formatting::prettyName(const UserID &uid)
 
 QString Formatting::prettyName(const UserID::Signature &sig)
 {
-    return prettyName(OpenPGP, sig.signerUserID(), sig.signerName(), sig.signerComment());
+    return prettyName(GpgME::OpenPGP, sig.signerUserID(), sig.signerName(), sig.signerComment());
 }
 
 //
@@ -191,7 +192,9 @@ QString Formatting::prettyEMail(const UserID::Signature &sig)
 
 QString Formatting::prettyEMail(const char *email_, const char *id)
 {
-    QString email, name, comment;
+    QString email;
+    QString name;
+    QString comment;
     if (email_ && KEmailAddress::splitAddress(QString::fromUtf8(email_),
                                               name, email, comment) == KEmailAddress::AddressOk) {
         return email;
@@ -209,7 +212,8 @@ namespace
 
 static QString protect_whitespace(QString s)
 {
-    static const QLatin1Char SP(' '), NBSP('\xA0');
+    static const QLatin1Char SP(' ');
+    static const QLatin1Char NBSP('\xA0');
     return s.replace(SP, NBSP);
 }
 
@@ -310,7 +314,7 @@ static QString make_red(const QString &txt)
 
 QString Formatting::toolTip(const Key &key, int flags)
 {
-    if (flags == 0 || (key.protocol() != CMS && key.protocol() != OpenPGP)) {
+    if (flags == 0 || (key.protocol() != GpgME::CMS && key.protocol() != GpgME::OpenPGP)) {
         return QString();
     }
 
@@ -318,7 +322,7 @@ QString Formatting::toolTip(const Key &key, int flags)
 
     QString result;
     if (flags & Validity) {
-        if (key.protocol() == OpenPGP || (key.keyListMode() & Validate)) {
+        if (key.protocol() == GpgME::OpenPGP || (key.keyListMode() & Validate)) {
             if (key.isRevoked()) {
                 result = make_red(i18n("Revoked"));
             } else if (key.isExpired()) {
@@ -353,7 +357,7 @@ QString Formatting::toolTip(const Key &key, int flags)
     }
 
     result += QLatin1String("<table border=\"0\">");
-    if (key.protocol() == CMS) {
+    if (key.protocol() == GpgME::CMS) {
         if (flags & SerialNumber) {
             result += format_row(i18n("Serial number"), key.issuerSerial());
         }
@@ -363,15 +367,18 @@ QString Formatting::toolTip(const Key &key, int flags)
     }
     if (flags & UserIDs) {
         const std::vector<UserID> uids = key.userIDs();
-        if (!uids.empty())
-            result += format_row(key.protocol() == CMS
+        if (!uids.empty()) {
+            result += format_row(key.protocol() == GpgME::CMS
                                  ? i18n("Subject")
                                  : i18n("User-ID"), prettyUserID(uids.front()));
-        if (uids.size() > 1)
-            for (auto it = uids.begin() + 1, end = uids.end(); it != end; ++it)
+        }
+        if (uids.size() > 1) {
+            for (auto it = uids.begin() + 1, end = uids.end(); it != end; ++it) {
                 if (!it->isRevoked() && !it->isInvalid()) {
                     result += format_row(i18n("a.k.a."), prettyUserID(*it));
                 }
+            }
+        }
     }
     if (flags & ExpiryDates) {
         result += format_row(i18n("Created"), time_t2string(subkey.creationTime()));
@@ -395,7 +402,7 @@ QString Formatting::toolTip(const Key &key, int flags)
         result += format_row(i18n("Fingerprint"), key.primaryFingerprint());
     }
     if (flags & OwnerTrust) {
-        if (key.protocol() == OpenPGP) {
+        if (key.protocol() == GpgME::OpenPGP) {
             result += format_row(i18n("Certification trust"), ownerTrustShort(key));
         } else if (key.isRoot()) {
             result += format_row(i18n("Trusted issuer?"),
@@ -455,7 +462,7 @@ namespace
 template <typename Container>
 QString getValidityStatement(const Container &keys)
 {
-    const bool allKeysAreOpenPGP = std::all_of(keys.cbegin(), keys.cend(), [] (const Key &key) { return key.protocol() == OpenPGP; });
+    const bool allKeysAreOpenPGP = std::all_of(keys.cbegin(), keys.cend(), [] (const Key &key) { return key.protocol() == GpgME::OpenPGP; });
     const bool allKeysAreValidated = std::all_of(keys.cbegin(), keys.cend(), [] (const Key &key) { return key.keyListMode() & Validate; });
     if (allKeysAreOpenPGP || allKeysAreValidated) {
         const bool someKeysAreBad = std::any_of(keys.cbegin(), keys.cend(), std::mem_fn(&Key::isBad));
@@ -538,6 +545,10 @@ static QString date2string(const QDate &date)
 {
     return QLocale().toString(date, QLocale::ShortFormat);
 }
+static QString localized_long_format(const QDate &date)
+{
+    return date.isValid() ? QLocale().toString(date, QLocale::LongFormat) : QString{};
+}
 
 template <typename T>
 QString expiration_date_string(const T &tee)
@@ -561,6 +572,11 @@ QString Formatting::dateString(time_t t)
     return date2string(time_t2date(t));
 }
 
+QString Formatting::accessibleDate(time_t t)
+{
+    return localized_long_format(time_t2date(t));
+}
+
 QString Formatting::expirationDateString(const Key &key)
 {
     return expiration_date_string(key.subkey(0));
@@ -591,6 +607,11 @@ QDate Formatting::expirationDate(const UserID::Signature &sig)
     return expiration_date(sig);
 }
 
+QString Formatting::accessibleExpirationDate(const Key &key)
+{
+    return key.subkey(0).neverExpires() ? i18n("no expiration") : localized_long_format(expirationDate(key));
+}
+
 QString Formatting::creationDateString(const Key &key)
 {
     return date2string(creation_date(key.subkey(0)));
@@ -621,16 +642,21 @@ QDate Formatting::creationDate(const UserID::Signature &sig)
     return creation_date(sig);
 }
 
+QString Formatting::accessibleCreationDate(const Key &key)
+{
+    return localized_long_format(creationDate(key));
+}
+
 //
 // Types
 //
 
-QString Formatting::displayName(Protocol p)
+QString Formatting::displayName(GpgME::Protocol p)
 {
-    if (p == CMS) {
+    if (p == GpgME::CMS) {
         return i18nc("X.509/CMS encryption standard", "S/MIME");
     }
-    if (p == OpenPGP) {
+    if (p == GpgME::OpenPGP) {
         return i18n("OpenPGP");
     }
     return i18nc("Unknown encryption protocol", "Unknown");
@@ -850,35 +876,35 @@ QString Formatting::signatureToString(const Signature &sig, const Key &key)
     const bool red   = (sig.summary() & Signature::Red);
     const bool valid = (sig.summary() & Signature::Valid);
 
-    if (red)
-        if (key.isNull())
+    if (red) {
+        if (key.isNull()) {
             if (const char *fpr = sig.fingerprint()) {
                 return i18n("Bad signature by unknown certificate %1: %2", QString::fromLatin1(fpr), QString::fromLocal8Bit(sig.status().asString()));
             } else {
                 return i18n("Bad signature by an unknown certificate: %1", QString::fromLocal8Bit(sig.status().asString()));
             }
-        else {
+        } else {
             return i18n("Bad signature by %1: %2", keyToString(key), QString::fromLocal8Bit(sig.status().asString()));
         }
 
-    else if (valid)
-        if (key.isNull())
+    } else if (valid) {
+        if (key.isNull()) {
             if (const char *fpr = sig.fingerprint()) {
                 return i18n("Good signature by unknown certificate %1.", QString::fromLatin1(fpr));
             } else {
                 return i18n("Good signature by an unknown certificate.");
             }
-        else {
+        } else {
             return i18n("Good signature by %1.", keyToString(key));
         }
 
-    else if (key.isNull())
+    } else if (key.isNull()) {
         if (const char *fpr = sig.fingerprint()) {
             return i18n("Invalid signature by unknown certificate %1: %2", QString::fromLatin1(fpr), QString::fromLocal8Bit(sig.status().asString()));
         } else {
             return i18n("Invalid signature by an unknown certificate: %1", QString::fromLocal8Bit(sig.status().asString()));
         }
-    else {
+    } else {
         return i18n("Invalid signature by %1: %2", keyToString(key), QString::fromLocal8Bit(sig.status().asString()));
     }
 }
@@ -892,10 +918,11 @@ QString Formatting::importMetaData(const Import &import, const QStringList &ids)
     const QString result = importMetaData(import);
     if (result.isEmpty()) {
         return QString();
-    } else
+    } else {
         return result + QLatin1Char('\n') +
                i18n("This certificate was imported from the following sources:") + QLatin1Char('\n') +
                ids.join(QLatin1Char('\n'));
+    }
 }
 
 QString Formatting::importMetaData(const Import &import)
@@ -908,15 +935,17 @@ QString Formatting::importMetaData(const Import &import)
     if (import.error().isCanceled()) {
         return i18n("The import of this certificate was canceled.");
     }
-    if (import.error())
+    if (import.error()) {
         return i18n("An error occurred importing this certificate: %1",
                     QString::fromLocal8Bit(import.error().asString()));
+    }
 
     const unsigned int status = import.status();
-    if (status & Import::NewKey)
+    if (status & Import::NewKey) {
         return (status & Import::ContainedSecretKey)
                ? i18n("This certificate was new to your keystore. The secret key is available.")
                : i18n("This certificate is new to your keystore.");
+    }
 
     QStringList results;
     if (status & Import::NewUserIDs) {
@@ -1105,7 +1134,7 @@ QString Formatting::complianceStringForKey(const GpgME::Key &key)
 {
     // There will likely be more in the future for other institutions
     // for now we only have DE-VS
-    if (complianceMode() == QLatin1String("de-vs")) {
+    if (Kleo::gnupgIsDeVsCompliant()) {
         if (uidsHaveFullValidity(key) && isKeyDeVs(key)) {
             return i18nc("%1 is a placeholder for the name of a compliance mode. E.g. NATO RESTRICTED compliant or VS-NfD compliant",
                          "May be used for %1 communication.", deVsString());
@@ -1121,8 +1150,7 @@ QString Formatting::complianceStringShort(const GpgME::Key &key)
 {
     const bool keyValidityChecked = (key.keyListMode() & GpgME::Validate);
     if (keyValidityChecked && Formatting::uidsHaveFullValidity(key)) {
-        if (complianceMode() == QLatin1String("de-vs")
-            && Formatting::isKeyDeVs(key)) {
+        if (Kleo::gnupgIsDeVsCompliant() && Formatting::isKeyDeVs(key)) {
             return QStringLiteral("★ ") + deVsString(true);
         }
         return i18nc("As in all user IDs are valid.", "certified");
@@ -1173,6 +1201,18 @@ QString Formatting::prettyID(const char *id)
     return ret;
 }
 
+QString Formatting::accessibleHexID(const char *id)
+{
+    static const QRegularExpression groupOfFourRegExp{QStringLiteral("(?:(.)(.)(.)(.))")};
+
+    QString ret;
+    ret = QString::fromLatin1(id);
+    if (!ret.isEmpty() && (ret.size() % 4 == 0)) {
+        ret = ret.replace(groupOfFourRegExp, QStringLiteral("\\1 \\2 \\3 \\4, ")).chopped(2);
+    }
+    return ret;
+}
+
 QString Formatting::origin(int o)
 {
     switch (o) {
